from pathlib import Path
from copy import deepcopy
from .keys import (
    BASE_PATH_KEY, BASE_LOG_DIR_KEY, TMP_DIR_KEY, LOG_TIMESTAMP_KEY,
    APPEND_LOG_KEY, CLI_LOG_LEVEL_KEY, FILE_LOG_LEVEL_KEY,
    LOCAL_CONFIG_FILE_KEY, LOG_TRACE_ENABLED_KEY, LOCAL_CFG_KEY,
    CONFIG_PACKAGE_DIR_KEY, CONFIG_PACKAGE_VERSION_KEY,
    SHOW_TRACEBACK_LOCALS_KEY, QRC_FILE_KEY, LOG_FUNC_CALL_ENABLED_KEY,
    STATUSBAR_LOG_LEVEL_KEY,
)

# defaults
DEFAULTS = {
    BASE_PATH_KEY: Path('.').resolve(),
    BASE_LOG_DIR_KEY: "logs",
    TMP_DIR_KEY: "${base_path:abs}",
    LOG_TIMESTAMP_KEY: True,
    APPEND_LOG_KEY: False,
    CLI_LOG_LEVEL_KEY: "info",  # can also use the int values
    FILE_LOG_LEVEL_KEY: "info",  # can also use the int values
    STATUSBAR_LOG_LEVEL_KEY: "info",  # can also use the int values
    LOG_TRACE_ENABLED_KEY: False,
    LOG_FUNC_CALL_ENABLED_KEY: False,
    SHOW_TRACEBACK_LOCALS_KEY: False,
    QRC_FILE_KEY: f'${{{CONFIG_PACKAGE_DIR_KEY}}}/gui/styles/assets/vibedark.qrc',  # noqa: E501

    LOCAL_CFG_KEY: {
        "theme": "vibedark",
        "default_width": 1080,
        "default_height": 720,
    },
    LOCAL_CONFIG_FILE_KEY: "~/.pyrandyos_local_config.jsonc",
}


def get_defaults(cls: type, app_global_defaults: dict = {},
                 app_local_defaults: dict = {}):
    tmp = deepcopy(DEFAULTS)

    from ..utils.stack import top_package_dir_path
    cfgpkgdir = top_package_dir_path()
    tmp[CONFIG_PACKAGE_DIR_KEY] = cfgpkgdir

    from ..version import __version__
    tmp[CONFIG_PACKAGE_VERSION_KEY] = __version__

    tmp.update(app_global_defaults)
    local: dict = tmp.get(LOCAL_CFG_KEY, {})
    local.update(app_local_defaults)
    tmp[LOCAL_CFG_KEY] = local
    return tmp



from os import environ
from pathlib import Path

from ..logging import log_func_call, DEBUGLOW2
from ..utils.expandvars import substitute_key, get_unresolved_keys
from ..utils.cfgdict import config_dict_get, config_dict_set
from ..utils.constants import IS_WIN32
from ..utils.casesafe import casesafe_value_in_container


@log_func_call(DEBUGLOW2, trace_only=True)
def _expand_nested(config: dict, key: str, value,
                   skip_expansion: str | list[str] | tuple[str] = None,
                   case_insensitive: bool = IS_WIN32):
    case = case_insensitive
    expand_key_recursively(config, key, skip_expansion, case_insensitive=case)
    return substitute_key(value, key,
                          config_dict_get(config, key,
                                          case_insensitive=case), case)


@log_func_call(DEBUGLOW2, trace_only=True)
def expand_key_recursively(config: dict, key: str,
                           skip_expansion: str | list[str] | tuple[str] = None,
                           fail: bool = False, set_value: bool = True,
                           case_insensitive: bool = IS_WIN32):
    case = case_insensitive
    skip_expansion = skip_expansion or ()
    if isinstance(skip_expansion, str):
        skip_expansion = (skip_expansion,)

    value = config_dict_get(config, key, case_insensitive=case)
    if isinstance(value, Path):
        value = str(value)
    while isinstance(value, str) and '$' in value:
        needed_keys = get_unresolved_keys(value)
        skipped = 0
        for k in needed_keys:
            if casesafe_value_in_container(skip_expansion, k, case):
                skipped += 1
                continue
            if '.' in k or casesafe_value_in_container(config, k, case):
                value = _expand_nested(config, k, value, skip_expansion,
                                       case)
            elif k in environ:
                value = substitute_key(value, k, environ[k])
            else:
                if fail:
                    raise KeyError(f'unknown key: {k}')
                else:
                    skipped += 1

        if skipped and skipped == len(needed_keys):
            break

    if set_value:
        config_dict_set(config, key, value, case)

    return value



from pyrandyos.gui.icons.iconfont import IconSpec, IconLayer, IconStateSpec
from pyrandyos.gui.icons.thirdparty.codicons import Codicons
from pyrandyos.gui.icons.thirdparty.codicons import names as codicon_names

ConfigLayer = IconLayer(Codicons, codicon_names.json,
                        x=0, y=0.075, scale=0.5)
SaveLayer = IconLayer(Codicons, codicon_names.save, y=-0.075)
SaveConfigIcon = IconSpec(IconStateSpec([
    SaveLayer,
    ConfigLayer,
]))


HomeLayer = IconLayer(Codicons, codicon_names.home,
                      x=0, y=0.1, scale=0.5)
SaveLayer = IconLayer(Codicons, codicon_names.save, y=-0.075)
SaveLocalConfigIcon = IconSpec(IconStateSpec([
    SaveLayer,
    HomeLayer,
]))



from ...logging import log_func_call
from ..qt import QGraphicsView, QGraphicsScene
from . import QtWidgetWrapper, GuiWidgetParentType


class GraphViewWidget(QtWidgetWrapper[QGraphicsView]):
    @log_func_call
    def __init__(self, parent: GuiWidgetParentType,
                 scene: QGraphicsScene = None):
        super().__init__(parent, scene)

    @log_func_call
    def create_qtobj(self, scene: QGraphicsScene = None):
        qtwin = self.gui_parent.qtobj

        scene = scene or QGraphicsScene(qtwin)
        self.scene = scene

        view = QGraphicsView(scene, qtwin)
        self.view = view
        return view
